C********1*********2*********3*********4*********5*********6*********7**
C   This module is thought to include subroutines provided by user, to
C   be used with the program FULLPROF, concerning "Microstructure".
C   For instance, hkl dependence of halfwidth parameters due to strains, etc.
C   Parameter for generalized strains and size effects are stored
C   in PAR(NPHT,MPAR), further parameters defined by user start
C   at PAR(NPHT,LASTMPAR-->MPAR). At present LASTMPAR=45
c   Strain and size parameters are using PAR(NPHT,18-->34).
C   Parameters 35-->37 are now used by the new asymmetry correction
C   function.
C   HKL-dependent asymmetry and shift for stacking faults and other
C   defects can be modeled by using up to three parameters stored
C   in AS=PAR(NPHT,38), and Shift1=PAR(NPHT,39) and Shift2=PAR(NPHT,40)
C   PAR(NPHT,41-42) are used for additional shape parameters.
C   PAR(NPHT,43-45) are used for additional U,V,W parameters.
C   Last update June 1995
C
C  WARNING!: not all the parts of this module have been fully tested.
C            It may be errors or some mistakes in the calculations.
C            Please notify the author any bug found in using these
C            routines!!!
C
C=======================================================================
      SUBROUTINE STRAIN(IX,H,K,L,IP,DST2,DVV,SQ,AL)
        INCLUDE 'ful0.inc'
        INCLUDE 'ful1.inc'
        INCLUDE 'ful2.inc'
        DIMENSION DVV(10),HH(3),AL(3,3)
        REAL*4 K,L
        DATA PRMUL/0.018203744/,G/0.017453293/
c       DATA PRMUL/0.018203744/,RAD/57.29577951/,G/0.017453293/
C
C     PRMUL = 1.0E-06 * 8Ln2 * (180/PI)**2
C----------------------------------------------------------------------
C    IP: number of the phase to which belongs the reflection HKL
C    DST: contribution to the broadening of reflection HKL due
C         to the particular model of strain
C    DVV(j): Derivative of DST2=DST*DST with respect to the strain
C            parameter E
C    SQ: 1/d2
C     As strains produce a tan(theta) dependence of the broadening
C    of Bragg reflections, the effect of the strain has been included
C    as the following formula:
C
C           H = sqrt {(U+DST2)tan2 + V tan + W}
C
C    this implies a gaussian distribution of strains
C
C      Fwhm(strain) = 2sqrt(2Ln2) Sigma(SQ)/SQ  * tan = DST*tan
C
C      Var(SQ)=Sigma(SQ)**2= Sum(i)Sum(j):Cov(ai,aj)(D(SQ)/Dai)(D(SQ)/Daj)
C--------------------------------------------------------------------
C  ISTR=0 + ISMO=1 -----> orth. strain in a tetragonal lattice
C   "     + ISMO=2 -----> strain along a
C   "     + ISMO=3 -----> the same (strain along one of the diagonals
C                           in the EQUIVALENT monoclinic double cell)
C   "     + ISMO=4 -----> the same ,but strain along BOTH diagonals
C                           in the EQUIVALENT monoclinic double cell)
C   "     + ISMO=5 -----> strain along a and b in an orthorhombic cell.
C   "     + ISMO=6 -----> strain along a,b,c with
C                         corr(a,b) = corr(a,c) = -1
C   "     + ISMO=7 -----> Uniaxial strains along a direction [S1,S2,S3]
C   "     + ISMO=8 -----> Strain for hexagonal symmetry in reciprocal space
C   "     + ISMO=9 -----> Strain adapted to LaNi5 phases (P.Thompson)
C   "     + ISMO=10-----> Strain for orthorhombic symmetry in reciprocal space
C   "     + ISMO=11-----> Strain for monoclinic   symmetry in reciprocal space
C
C  ISTR=1 ----------------> generalized strains
C-------------------------------------------------------------------
        if(IX.GT.IRS)
     +  CALL STOP_COND('  Too many reflections passed to STRAIN!',1)
        DST2=0
        DO 1 J=1,10
   1    DVV(J)=0.0
        IF(ISTR(IP).NE.0) GOTO 1111
        IF(ISMO(IP).EQ.0) RETURN
        GOTO(200,300,400,500,600,700,800,1000,1100,12,13,100),ISMO(IP)
  100   RETURN
C
C  Orthorhombic strains in a tetragonal lattice with the constrain:
C
C   a(loc)=a0(1+dx)  b(loc)=b0(1-dx) c=c0
C  {rms(dx): strain parameter (adimensional)}
C

 200    DS=2.0*ABS(H*H-K*K)/(SAVE(IP,1)**2)/SQ
        DST=DS*PAR(IP,18)
        DVV(1)=2.*DST*DS*PRMUL
        DST2=DST*DST*PRMUL
        RETURN
C.....................................................................
C              Strains in an orthorhombic lattice along a. (M.Medarde)
C    sqrt(DST)=(2h*h/(a*a)/SQ)*dx  (dx=da/a strain parameter adim.)
C
C    (to have the true value, you must divide the strain
C               refined parameter by 1000)
C......................................................................
C
  300   DS=2.0*H*H/(SAVE(IP,1)*SAVE(IP,1))/SQ
        DST=DS*PAR(IP,18)
        DVV(1)=2.*PRMUL*DST*DS
        DST2=DST*DST*PRMUL
        RETURN
C........................................................................
C    strains in an equivalent monoclinic double-cell along (M.Medarde)
C                  ONE of the  diagonals
C  sqrt(DST)= ((h+l/a*cos(beta/2))**2)*dx/(2*SQ) (dx=da/a strain parameter)
C
C             warning!  P21/B -> H+K
C......................................................................
C
  400   DS=ABS(H+K)/(SAVE(IP,1)*COS(SAVE(IP,6)*G/2.0))
        DS=DS*DS/(2.*SQ)
        DST=DS*PAR(IP,18)
        DVV(1)=2.*PRMUL*DST*DS
        DST2=DST*DST*PRMUL
        RETURN
C........................................................................
C    strains in an equivalent monoclinic double-cell along  (M.Medarde)
C                  BOTH  diagonals
C  DST2*(16*SQ) =  ((h+k/a*cos(gamma/2))**2)*dx*dx  +
C                  ((h-k/b*sin(gamma/2))**2)*dy*dy
C
C           (dx=da+db/a, dy=da+db/b  strain parameters)
C
C                 warning! P21/B -> H+K
C......................................................................
C
  500   DS1 =  ABS(H+K)/(SAVE(IP,1)*COS(SAVE(IP,6)*G/2.0))
        DS1 =  (DS1*DS1*DS1*DS1*PAR(IP,18)*PAR(IP,18))/(16.*SQ*SQ)
        DS2 =  ABS(H-K)/(SAVE(IP,2)*SIN(SAVE(IP,6)*G/2.0))
        DS2 =  (DS2*DS2*DS2*DS2*PAR(IP,19)*PAR(IP,19))/(16.*SQ*SQ)
         DVV(1) = (PRMUL*2.*DS1)/PAR(IP,18)
         DVV(2) = (PRMUL*2.*DS2)/PAR(IP,19)
        DST2=  PRMUL*(DS1+DS2)
        RETURN
C
C...................................................................
C      Strains along a and/or b in an orthorrombic cell  (M.Medarde)
C
C  DST2 = 4.PRMUL.{dx**2.(h/a)**4 + dy**2.(k/b)**4 +
C       + 2.corr(ab).dx.dy.(h/a)**2.(k/b)**2}/SQ.SQ
C
C..................................................................
C
  600     AH = H/SAVE(IP,1)
         BBK = K/SAVE(IP,2)
      DVV(1) = 2.0*PAR(IP,18)*AH*AH*AH*AH
      DVV(1) = DVV(1) + 2.0*PAR(IP,19)*PAR(IP,20)*AH*AH*BBK*BBK
      DVV(1) = DVV(1)*4.0*PRMUL/(SQ*SQ)
      DVV(2) = 2.0*PAR(IP,19)*BBK*BBK*BBK*BBK
      DVV(2) = DVV(2) + 2.0*PAR(IP,18)*PAR(IP,20)*AH*AH*BBK*BBK
      DVV(2) = DVV(2)*4.0*PRMUL/(SQ*SQ)
      DVV(3) = 2.0*PAR(IP,18)*PAR(IP,19)*AH*AH*BBK*BBK
      DVV(3) = DVV(3)*4.0*PRMUL/(SQ*SQ)
C
        DS =      PAR(IP,18)*PAR(IP,18)*AH*AH*AH*AH
        DS = DS + PAR(IP,19)*PAR(IP,19)*BBK*BBK*BBK*BBK
        DS = DS + 2.0*PAR(IP,18)*PAR(IP,19)*PAR(IP,20)*AH*AH*BBK*BBK
      DST2 = DS*4.0*PRMUL/(SQ*SQ)
        RETURN
C
C...................................................................
C      Strains along a,b,c in an orthorrombic cell with   (M.Medarde)
C        corr(a,b)=-1 corr(a,c)=1   corr(b,c)=0
C
C  DST2 = 4.PRMUL.{dx**2.(h/a)**4 + dy**2.(k/b)**4 +
C       + 2.corr(ab).dx.dy.(h/a)**2.(k/b)**2}/SQ.SQ
C
C..................................................................
C
  700     AH = H/SAVE(IP,1)
         BBK = K/SAVE(IP,2)
          CL = L/SAVE(IP,3)
      DVV(1) = 2.0*PAR(IP,18)*AH*AH*AH*AH
      DVV(1) = DVV(1) - 2.0*PAR(IP,19)*AH*AH*BBK*BBK
      DVV(1) = DVV(1) + 2.0*PAR(IP,20)*AH*AH*CL*CL
      DVV(1) = DVV(1)*4.0*PRMUL/(SQ*SQ)
      DVV(2) = 2.0*PAR(IP,19)*BBK*BBK*BBK*BBK
      DVV(2) = DVV(2) - 2.0*PAR(IP,18)*AH*AH*BBK*BBK
      DVV(2) = DVV(2) + 2.0*PAR(IP,20)*AH*AH*CL*CL
      DVV(2) = DVV(2)*4.0*PRMUL/(SQ*SQ)
      DVV(3) = 2.0*PAR(IP,20)*CL*CL*CL*CL
      DVV(3) = DVV(3) - 2.0*PAR(IP,18)*AH*AH*BBK*BBK
      DVV(3) = DVV(3) + 2.0*PAR(IP,19)*AH*AH*CL*CL
      DVV(3) = DVV(3)*4.0*PRMUL/(SQ*SQ)
C
        DS =      PAR(IP,18)*PAR(IP,18)*AH*AH*AH*AH
        DS = DS + PAR(IP,19)*PAR(IP,19)*BBK*BBK*BBK*BBK
        DS = DS + PAR(IP,20)*PAR(IP,20)*CL*CL*CL*CL
        DS = DS - 2.0*PAR(IP,18)*PAR(IP,19)*AH*AH*BBK*BBK
        DS = DS + 2.0*PAR(IP,18)*PAR(IP,20)*AH*AH*CL*CL
        DST2 = DS*4.0*PRMUL/(SQ*SQ)
        RETURN
C------------------------------------------------------------
C Uniaxial strains along [S1,S2,S3]=PST    (JRC)
C------------------------------------------------------------
  800 DS=0.0
      HH(1)=H
      HH(2)=K
      HH(3)=L
      DO 33 I=1,3
      DO 33 J=I,3
   33 DS=DS+AL(I,J)*(PST(IP,I)*HH(J)+PST(IP,J)*HH(I))/2.
      DS=DS*DS/(PST(IP,4)*SQ)
      DST=DS*PAR(IP,18)
      DVV(1)=2.*DST*DS*PRMUL
      DST2=DST*DST*PRMUL
      RETURN
c
c  General Strains  in an Hexagonal lattice. Covariance Matrix in
c  reciprocal lattice: M = A (h2 + k2 + hk) + C l2.
c  Parameters SAA, SCC, CAC
C
 1000 s0=h*h+k*k+h*k
      s1=s0*s0
      s2=l*l*l*l
      s3=2.*s0*l*l
      DST2=PAR(IP,18)*PAR(IP,18)*S1+PAR(IP,19)*PAR(IP,19)*S2+
     +     PAR(IP,18)*PAR(IP,19)*PAR(IP,20)*S3
      PR=PRMUL/(SQ*SQ)
      DST2=PR*DST2
      DVV(1)=PR*(2.*PAR(IP,18)*S1+PAR(IP,19)*PAR(IP,20)*S3)
      DVV(2)=PR*(2.*PAR(IP,19)*S2+PAR(IP,18)*PAR(IP,20)*S3)
      DVV(3)=PR*PAR(IP,18)*PAR(IP,19)*S3
      RETURN
c
c  Strains for LaNi5 and related compounds (P. Thompson formula)
c
 1100 s1=h*h+k*k+l*l
      s1=s1*s1
      s2=l*l*l*l/s1
      s3=h*h*k*k+k*k*l*l/s1
      s4=h*h*k*k/s1
      DST2=PAR(IP,18)+S2*PAR(IP,19)+S3*PAR(IP,20)+S4*PAR(IP,21)
      DVV(1)=1.
      DVV(2)=S2
      DVV(3)=S3
      DVV(4)=S4
      return
c
c  General Strains  in an orthorhombic lattice. Covariance Matrix in
c  reciprocal lattice: M = A h2 + B k2 + C l2.
c  Parameters SAA, SBB, SCC, CAB, CAC, CBC
C
   12 s1=h*h*h*h
      s2=k*k*k*k
      s3=l*l*l*l
      s4=2*h*h*k*k
      s5=2*h*h*l*l
      s6=2*k*k*l*l
      DST2=PAR(IP,18)*PAR(IP,18)*S1+PAR(IP,19)*PAR(IP,19)*S2+
     +     PAR(IP,20)*PAR(IP,20)*S3+PAR(IP,18)*PAR(IP,19)*PAR(IP,21)*S4+
     +     PAR(IP,18)*PAR(IP,20)*PAR(IP,22)*S5+
     +     PAR(IP,19)*PAR(IP,20)*PAR(IP,23)*S6
      PR=PRMUL/(SQ*SQ)
      DST2=PR*DST2
      DVV(1)=PR*(2.*PAR(IP,18)*S1+PAR(IP,19)*PAR(IP,21)*S4+
     +           PAR(IP,20)*PAR(IP,22)*S5)
      DVV(2)=PR*(2.*PAR(IP,19)*S2+PAR(IP,18)*PAR(IP,21)*S4+
     +           PAR(IP,20)*PAR(IP,23)*S6)
      DVV(3)=PR*(2.*PAR(IP,20)*S3+PAR(IP,18)*PAR(IP,22)*S5+
     +           PAR(IP,19)*PAR(IP,23)*S6)
      DVV(4)=PR*PAR(IP,18)*PAR(IP,19)*S4
      DVV(5)=PR*PAR(IP,18)*PAR(IP,20)*S5
      DVV(6)=PR*PAR(IP,19)*PAR(IP,20)*S6
      RETURN
c
c  General Strains  in a monoclinic lattice. Covariance Matrix in
c  reciprocal lattice: M = A h2 + B k2 + C l2 + D hk.
c  Parameters Saa, Sbb, Scc, Sdd, Cab, Cac, Cbc, Ccd
c
   13 s1=h*h*h*h
      s2=k*k*k*k
      s3=l*l*l*l
      s4=h*h*k*k
      s5=2*h*h*k*k   ! 2.s4
      s6=2*h*h*l*l
      s7=2*k*k*l*l
      s8=2*h*k*l*l
      DST2=PAR(IP,18)*PAR(IP,18)*S1+PAR(IP,19)*PAR(IP,19)*S2+
     +     PAR(IP,20)*PAR(IP,20)*S3+PAR(IP,21)*PAR(IP,21)*S4+
     +     PAR(IP,18)*PAR(IP,19)*PAR(IP,22)*S5+
     +     PAR(IP,18)*PAR(IP,20)*PAR(IP,23)*S6+
     +     PAR(IP,19)*PAR(IP,20)*PAR(IP,24)*S7+
     +     PAR(IP,20)*PAR(IP,21)*PAR(IP,25)*S8
      PR=PRMUL/(SQ*SQ)
      DST2=PR*DST2
      DVV(1)=PR*(2.*PAR(IP,18)*S1+PAR(IP,19)*PAR(IP,22)*S5+
     +           PAR(IP,20)*PAR(IP,23)*S6)
      DVV(2)=PR*(2.*PAR(IP,19)*S2+PAR(IP,18)*PAR(IP,22)*S5+
     +           PAR(IP,20)*PAR(IP,24)*S7)
      DVV(3)=PR*(2.*PAR(IP,20)*S3+PAR(IP,18)*PAR(IP,23)*S6+
     +           PAR(IP,19)*PAR(IP,24)*S7+PAR(IP,21)*PAR(IP,25)*S8)
      DVV(4)=PR*(2.*PAR(IP,21)*S4+PAR(IP,20)*PAR(IP,25)*S8)
      DVV(5)=PR*PAR(IP,18)*PAR(IP,19)*S5
      DVV(6)=PR*PAR(IP,18)*PAR(IP,20)*S6
      DVV(7)=PR*PAR(IP,19)*PAR(IP,20)*S7
      DVV(8)=PR*PAR(IP,20)*PAR(IP,21)*S8
      RETURN
 1111 CONTINUE
      IF(NUMSPG(IP).GE.6.AND.NUMSPG(IP).LE.12) GOTO 900
CC*********************************************************************
C
C                 GENERAL FORMULAE FOR STRAINS
C     DERIVATIVES OF a,b,c,alpha,beta,gamma  (direct cell parameters!!)
C
C**********************************************************************
C
C----------------------------------------------------------------------
C           GENERALIZED STRAINS ----- MONOCLINIC CASE  (M.Medarde&JRC)
C       Strain parameters are sigma(x) and the argument
C         (in degrees) of sin(Pxy) (x,y=a,b,c,beta)
C
C           cov(a,b) = sigma(a) * sigma(b) * corr(ab)
C                for us, corr(a,b)= sin(Pab)
C
C  (to have the true values, you must divide the FOUR strain
C                 refined parameters by 1000)
C----------------------------------------------------------------------
C
          AA = SAVE(IP,1)
           B = SAVE(IP,2)
           C = SAVE(IP,3)
          SI = SIN(SAVE(IP,5)*G)
          CO = COS(SAVE(IP,5)*G)
          TA = TAN(SAVE(IP,5)*G)
         S22 = SIN(G*PAR(IP,22))
         S23 = SIN(G*PAR(IP,23))
         S24 = SIN(G*PAR(IP,24))
         S25 = SIN(G*PAR(IP,25))
         S26 = SIN(G*PAR(IP,26))
         S27 = SIN(G*PAR(IP,27))
         C22 = COS(G*PAR(IP,22))
         C23 = COS(G*PAR(IP,23))
         C24 = COS(G*PAR(IP,24))
         C25 = COS(G*PAR(IP,25))
         C26 = COS(G*PAR(IP,26))
         C27 = COS(G*PAR(IP,27))
C
         DV1 = (-2.0*H*H)/(AA*AA*AA)
         DV1 = (DV1 + ((2.0*H*L*CO)/(AA*AA*C)))/(SI*SI)
         DV2 = (-2.0*K*K)/(B*B*B)
         DV3 = (-2.0*L*L)/(C*C*C)
         DV3 = (DV3 + ((2.0*H*L*CO)/(AA*C*C)))/(SI*SI)
         DV4 =  ((K*K)/(B*B)) + ((H*L)/(AA*C)) - SQ
         DV4 = DV4*2.0*G/TA
C
      DVV(5) = PRMUL*2.0*DV1*DV2*PAR(IP,18)*PAR(IP,19)
      DVV(5) = DVV(5)*C22*G/(SQ*SQ)
      DVV(6) = PRMUL*2.0*DV1*DV3*PAR(IP,18)*PAR(IP,20)
      DVV(6) = DVV(6)*C23*G/(SQ*SQ)
      DVV(7) = PRMUL*2.0*DV1*DV4*PAR(IP,18)*PAR(IP,21)
      DVV(7) = DVV(7)*C24*G/(SQ*SQ)
      DVV(8) = PRMUL*2.0*DV2*DV3*PAR(IP,19)*PAR(IP,20)
      DVV(8) = DVV(8)*C25*G/(SQ*SQ)
      DVV(9) = PRMUL*2.0*DV2*DV4*PAR(IP,19)*PAR(IP,21)
      DVV(9) = DVV(9)*C26*G/(SQ*SQ)
      DVV(10)= PRMUL*2.0*DV3*DV4*PAR(IP,20)*PAR(IP,21)
      DVV(10)= DVV(10)*C27*G/(SQ*SQ)
C
      DVV(1) = PAR(IP,18)*DV1 + PAR(IP,19)*DV2*S22
      DVV(1) = DVV(1) + PAR(IP,20)*DV3*S23 + PAR(IP,21)*DV4*S24
      DVV(1) = PRMUL*2.0*DVV(1)/(SQ*SQ)
      DVV(2) = PAR(IP,18)*DV1*S22 + PAR(IP,19)*DV2
      DVV(2) = DVV(2) + PAR(IP,20)*DV3*S25 + PAR(IP,21)*DV4*S26
      DVV(2) = PRMUL*2.0*DVV(2)/(SQ*SQ)
      DVV(3) = PAR(IP,18)*DV1*S23 + PAR(IP,19)*DV2*S25
      DVV(3) = DVV(3) + PAR(IP,20)*DV3 + PAR(IP,21)*DV4*S27
      DVV(3) = PRMUL*2.0*DVV(3)/(SQ*SQ)
      DVV(4) = PAR(IP,18)*DV1*S24 + PAR(IP,19)*DV2*S26
      DVV(4) = DVV(4) + PAR(IP,20)*DV3*S27 + PAR(IP,21)*DV4
      DVV(4) = PRMUL*2.0*DVV(4)/(SQ*SQ)
C
         DST =       PAR(IP,18)*PAR(IP,18)*DV1*DV1
         DST = DST + PAR(IP,19)*PAR(IP,19)*DV2*DV2
         DST = DST + PAR(IP,20)*PAR(IP,20)*DV3*DV3
         DST = DST + PAR(IP,21)*PAR(IP,21)*DV4*DV4
         DST = DST + PAR(IP,18)*PAR(IP,19)*DV1*DV2*S22*2.0
         DST = DST + PAR(IP,18)*PAR(IP,20)*DV1*DV3*S23*2.0
         DST = DST + PAR(IP,18)*PAR(IP,21)*DV1*DV4*S24*2.0
         DST = DST + PAR(IP,19)*PAR(IP,20)*DV2*DV3*S25*2.0
         DST = DST + PAR(IP,19)*PAR(IP,21)*DV2*DV4*S26*2.0
         DST = DST + PAR(IP,20)*PAR(IP,21)*DV3*DV4*S27*2.0
         DST2= PRMUL*DST/(SQ*SQ)
      RETURN
C
C------------------------------------------------------------------
C            GENERALIZED STRAINS ----- HEXAGONAL CASE   (M.Medarde&JRC)
C       Strain parameters are sigma(x) and the argument
C         (in degrees) of sin(Pxy) (x,y=a,b,c,gamma)
C
C           cov(a,b) = sigma(a) * sigma(b) * corr(ab)
C                for us, corr(a,b)= sin(Pab)
C
C  (to have the true values, you must divide the FOUR FIRST refined
C          strain parameters by 100)
C------------------------------------------------------------------
C
  900         AA = SAVE(IP,1)
               C = SAVE(IP,3)
             S20 = SIN(G*PAR(IP,20))
             C20 = COS(G*PAR(IP,20))
             VV1 =-(H*H+K*K+H*K)*8.0/(3.0*AA*AA*AA)
             VV2 = -2.0*L*L/(C*C*C)
          DVV(3) = PRMUL*2.0*PAR(IP,18)*PAR(IP,19)*VV1*VV2*C20
          DVV(3) = DVV(3)*G/(SQ*SQ)
          DVV(1) = PAR(IP,18)*VV1*VV1 + PAR(IP,19)*VV1*VV2*S20
          DVV(1) = DVV(1)*2.0*PRMUL/(SQ*SQ)
          DVV(2) = PAR(IP,19)*VV2*VV2 + PAR(IP,18)*VV1*VV2*S20
          DVV(2) = DVV(2)*2.0*PRMUL/(SQ*SQ)
C
        DST =       PAR(IP,18)*PAR(IP,18)*VV1*VV1
        DST = DST + PAR(IP,19)*PAR(IP,19)*VV2*VV2
        DST = DST + 2.0*PAR(IP,18)*PAR(IP,19)*VV1*VV2*S20
        DST2= PRMUL*DST/(SQ*SQ)
        RETURN
        END
C********1*********2*********3*********4*********5*********6*********7**
      SUBROUTINE SIZEf(IX,ICX,H,K,L,IP,DSIZ,DVS,SQ,AL)
        INCLUDE 'ful0.inc'
        INCLUDE 'ful1.inc'
        INCLUDE 'ful2.inc'
        DIMENSION DVS(6),HH(3),AL(3,3)
        REAL K,L,K2,L2,KL
        DATA RAD/57.29577951/
C----------------------------------------------------------------------
C    IX: ordinal number of reflection HKL in IREFS(IX)
C    IP: number of the phase to which belong the reflection HKL
C    DSIZ: contribution to the broadening of reflection HKL due
C         to the particular model of particle size
C    DVS(j): Derivative of DSIZ with respect to the size parameter SIZ
C    SQ: 1/d2
C     As size effects produce a 1/cos(theta) dependence of the broadening
C    of Bragg reflections, the effect of the size has been included
C    as contributing to the parameter Y(VLOR) of the Thompson-Cox-Hasting
C    formula for lorentzian broadening
C
C           Hlorentz = X*tan(theta)+{Y+DSIZ)}/cos(theta)
C--------------------------------------------------------------------
C  ISTR=0 + ISI=1 -----> Platelet-like particles
C   "     + ISI=-1-----> Needle-like particles
C   "     + ISI=2 -----> --- (00L)
C   "     + ISI=3 -----> --- (0K0)
C   "     + ISI=4 -----> --- (H00)
C   "     + ISI=5 -----> --- (HK0)
C   "     + ISI=6 -----> --- (H0L)
C   "     + ISI=7 -----> --- (0KL)
C   "     + ISI=8 -----> correlation length for satellites
C   "     + ISI=9 -----> H=2n+1 and K=2m+1
C   "     + ISI=10-----> H0L (H+L=2n)
C   "     + ISI=11-----> HKL (except HHL)
C   "     + ISI=12-----> HKL (h=2n+1 )
C   "     + ISI=13-----> HKL (h=2n   ) and (k=2n+1)
C   "     + ISI=14-----> HKL (h=2n+1 or k=2m+1 and l=2n+1)
C  ISTR=2 ----------------> generalized size parameters
C-------------------------------------------------------------------
      DSIZ=0.0
      DO 1 J=1,6
   1  DVS(J)=0.0
      IF(ISTR(IP).GE.2) GOTO 2000
      IF(ISI(IP).EQ.0) RETURN
C           1   2   3   4   5   6   7   8    9  10  11  12  13  14
      GOTO(200,300,400,500,600,700,800,1100,900,910,920,930,940,950),
     +     iabs(ISI(IP))
c-------------------------------------------------------------------
c Platelet-Needle particles. Instead of using an exact profile shape
c the lorentzian component of the assumed Voigt function is allowed
c to vary according to the expression:
c    HL=(Y+Dsiz*cos(q))/cos(theta) or   HL=(Y+Dsiz*sin(q))/cos(theta)
C where Y is the isotropic lorentzian-like size parameter    (JRC)
c-------------------------------------------------------------------
  200 DS=0.0
      HH(1)=H
      HH(2)=K
      HH(3)=L
      DO 33 I=1,3
      DO 33 J=I,3
   33 DS=DS+AL(I,J)*(PSZ(IP,I)*HH(J)+PSZ(IP,J)*HH(I))/2.
      DS=abs(DS)/SQRT(PSZ(IP,4)*SQ)
      IF(ISI(IP).EQ.1) THEN
       DSIZ=DS*PAR(IP,29)
       DVS(1)=DS
      ELSE
       DS=SQRT(1.-DS*DS)
       DSIZ=DS*PAR(IP,29)
       DVS(1)=DS
      ENDIF
      RETURN
  300 IF(H.EQ.0.AND.K.EQ.0.AND.L.NE.0) GOTO 1000
      RETURN
  400 IF(H.EQ.0.AND.K.NE.0.AND.L.EQ.0) GOTO 1000
      RETURN
  500 IF(H.NE.0.AND.K.EQ.0.AND.L.EQ.0) GOTO 1000
      RETURN
  600 IF(H.NE.0.AND.K.NE.0.AND.L.EQ.0) GOTO 1000
      RETURN
  700 IF(H.NE.0.AND.K.EQ.0.AND.L.NE.0) GOTO 1000
      RETURN
  800 IF(H.EQ.0.AND.K.NE.0.AND.L.NE.0) GOTO 1000
      RETURN
  900 IF(mod(int(H),2).NE.0.AND.mod(int(K),2).NE.0) GOTO 1000
      RETURN
  910 IF(mod(int(H+L),2).eq.0.AND.int(K).eq.0) GOTO 1000
      RETURN
  920 IF(.not.(int(H).eq.int(K))) GOTO 1000
      RETURN
  930 IF(mod(int(H),2).ne.0) GOTO 1000
      RETURN
  940 IF(mod(int(H),2).eq.0.AND.mod(int(K),2).NE.0) GOTO 1000
      RETURN
  950 IF( (mod(int(H),2).NE.0.or.mod(int(K),2).NE.0) .and.
     +                mod(int(L),2).NE.0) GOTO 1000
      RETURN
 1100 IF(IHKL(IX).NE.0) GOTO 1000
      RETURN
c 1000 CTE=LAMDA(ICX)*RAD/1000
 1000 DSIZ=PAR(IP,29)
      DVS(1)=1.0
      RETURN
 2000 CTE=LAMDA(ICX)*SQRT(SQ)*RAD/1000
      H2=H*H
      K2=K*K
      L2=L*L
      HK=2.0*H*K
      HL=2.0*H*L
      KL=2.0*K*L
      QUAD=PAR(IP,29)*H2+PAR(IP,30)*K2+PAR(IP,31)*L2
     *    +PAR(IP,32)*HK+PAR(IP,33)*HL+PAR(IP,34)*KL
      IF(QUAD.GT.0.0) THEN
      DSIZ=CTE/SQRT(QUAD)
      CTEP=-0.5*CTE/(QUAD**1.5)
       ELSE
      CALL STOP_COND('  Size quadratic form non positive !!',1)
      ENDIF
      DVS(1)=CTEP*H2
      DVS(2)=CTEP*K2
      DVS(3)=CTEP*L2
      DVS(4)=CTEP*HK
      DVS(5)=CTEP*HL
      DVS(6)=CTEP*KL
      RETURN
      END
C********1*********2*********3*********4*********5*********6*********7**
      SUBROUTINE COMSIZE(KIND)
      include 'ful0.inc'
      GOTO (1,2,3,4,5,6,7,8,9,10,11) iabs(KIND)
    1 if(kind.gt.0) then
      WRITE(i_out,'(A/A/A)')
     + ' => Platelet-like size effect: Lorentzian FWHM'
     +,'    modeled as HL*cos(theta)=Y + e cos(Phi), Y is the isotropic
     +part','    e is the anisotropic component and Phi=[hkl]^[uvw]'
      else
      WRITE(i_out,'(A/A/A)')
     + ' => Needle-like size effect: Lorentzian FWHM'
     +,'    modeled as HL*cos(theta)=Y + e sin(Phi), Y is the isotropic
     +part','    e is the anisotropic component and Phi=[hkl]^[uvw]'
      endif
      return
    2 WRITE(i_out,'(a)')' => Only 00L reflections are broadened by size'
      return
    3 WRITE(i_out,'(a)')' => Only 0K0 reflections are broadened by size'
      return
    4 WRITE(i_out,'(a)')' => Only H00 reflections are broadened by size'
      return
    5 WRITE(i_out,'(a)')' => Only HK0 reflections are broadened by size'
      return
    6 WRITE(i_out,'(a)')' => Only H0L reflections are broadened by size'
      return
    7 WRITE(i_out,'(a)')' => Only 0KL reflections are broadened by size'
      return
    8 WRITE(i_out,'(a)')
     +  ' => Only satellites are broadened by size effects'
      return
    9 WRITE(i_out,'(a)')
     +  ' => Only h=2n+1,k=2m+1 are broadened by size effects'
   10 WRITE(i_out,'(a)')
     +  ' => Only (H0L) wiht H+L=2n are broadened by size effects'
   11 WRITE(i_out,'(a)')
     +  ' => Only (HKL) with H<>K are broadened by size effects'
      return
      END
C************************************************************************
      SUBROUTINE COMSTRA(KIND)
      include 'ful0.inc'
      GOTO (1,2,3,4,5,6,7,8,9,10,11) KIND
    1 WRITE(i_out,'(A/A/a/)')
     +  ' => Orth. strain in a tetragonal lattice ',
     +  '    Gaussian FWHM=2.0*P*e*ABS(H*H-K*K)/(a*a)/SQ',
     +  '               P = 180/pi*sqrt(8 Ln2) 1.E-03 tan(theta)'
      return
    2 WRITE(i_out,'(A/A/A/)') ' => Strain along a-axis ',
     +  '    Gaussian FWHM=2.0*P*e*H*H/(a*a)/SQ',
     +  '         P = 180/pi*sqrt(8 Ln2) 1.E-03 tan(theta)'
      return
    3 WRITE(i_out,'(A/A/A/)')
     +  ' => Strain along [110] in a monoclinic P112/m lattice ',
     +  '    Gaussian FWHM=P*e*{(H+K)/(a*cos(gam/2))}^2/(2.SQ)',
     +  '               P = 180/pi*sqrt(8 Ln2) 1.E-03 tan(theta)'
      return
    4 WRITE(i_out,'(A/A/A/a/a/)')
     +  ' => Strain along [1,1,0] and [1,-1,0] in a P112/m lattice ',
     +  '    Gaussian FWHM^2= D1^2 + D2^2,  where:',
     +  '      D1=P*e1*{(H+K)/(a*cos(gam/2))}^2/(2.SQ)',
     +  '      D2=P*e2*{(H-K)/(b*sin(gam/2))}^2/(2.SQ)',
     +  '      P = 180/pi*sqrt(8 Ln2) 1.E-03 tan(theta)'
      return
    5 WRITE(i_out,'(A/A/A/a/a/)')
     +  ' => Strain along a and/or b in an orthorrombic lattice ',
     +  '    Gaussian FWHM^2= 4.P^2.{sx**2.(h/a)**4 + sy**2.(k/b)**4 +',
     +  '       +2.corr(ab).sx.sy.(h/a)**2.(k/b)**2}/SQ.SQ ,  where:',
     +  '    the refined parameters are: sx,sy and corr(ab)',
     +  '    P = 180/pi*sqrt(8 Ln2) 1.E-03 tan(theta)'
      return
    6 WRITE(i_out,'(A/A/A/a/a/a/)')
     +  ' => Strain along a, b and c in an orthorrombic lattice ',
     +  '    with corr(a,b) = corr(a,c) = -1 and corr(b,c)=0',
     +  '    Gaussian FWHM^2= 4.P^2.{sx**2.(h/a)**4 + sy**2.(k/b)**4 +',
     +  '       +sz**2.(l/c)**4}/SQ.SQ ,  where:',
     +  '    the refined parameters are: sx,sy and sz',
     +  '    P = 180/pi*sqrt(8 Ln2) 1.E-03 tan(theta)'
      return
    7 WRITE(i_out,'(A/A/A/a/)')
     +  ' => Uniaxial strain along a direction [S1,S2,S3] ',
     +  '    Gaussian FWHM= P.E.cos(Phi)',
     +  '    where E is the refined parameter and Phi=[hkl]^[S1,S2,S3]',
     +  '    P = 180/pi*sqrt(8 Ln2) 1.E-03 tan(theta)'
      return
    8 WRITE(i_out,'(A/A/A/a/a/a/)')
     +  ' => Strain adapted for hexagonal lattices   ',
     +  '    3 strain parameters corresponding to: ',
     +  '    Saa=Sigma(A), Scc=Sigma(C) and Cac=Correl(A,C)',
     +  '  ->    SQ = A (h^2+k^2+hk) + C l^2 = A m^2 + C l^2',
     +  '  -> Var(SQ)= Saa^2 m^4 + Scc^2 l^4 + 2 Saa Scc Cac m^2 l^2 ',
     +  ' FW^2= Var(SQ)(P/SQ)^2 {P=180/pi*sqrt(8Ln2) 1.E-03 tan(theta)}'
      return
    9 WRITE(i_out,'(A/A/A/a/a/a/a/)')
     +  ' => Strain adapted for compounds LaNi5-like. Formula by:  ',
     +  '    P.Thompson et al, J.Less Comm Met 129, 105-114 (1987) ',
     +  '    Four param.: Gaussian FWHM=Sqrt(Sumj[S(j)]) tan(theta)',
     +  '    -> S(1) =                                   x   Par(1)',
     +  '    -> S(2) =  l^4             /(h^2+k^2+l^2)   x   Par(2)',
     +  '    -> S(3) = (h^2 k^2+k^2 l^2)/(h^2+k^2+l^2)   x   Par(3)',
     +  '    -> S(4) =  h^2 k^2         /(h^2+k^2+l^2)   x   Par(4)'
      return
   10 WRITE(i_out,'(A/A/A/a/a/a/a/a/a/)')
     +  ' => Strain adapted for Orthogonal lattices   ',
     +  '    6 strain parameters corresponding to :',
     +  '    Saa=Sigma(A), Sbb, Scc, Cab, Cac and Cbc=Correl(B,C) ',
     +  '  ->     SQ = A h^2+ B k^2 + C l^2 ',
     +  '  -> Var(SQ)= Saa^2 h^4 + Sbb^2 k^4 + Scc^2 l^4 + ',
     +  '              2 Saa Sbb Cab h^2 k^2 +',
     +  '              2 Saa Scc Cac h^2 l^2 +',
     +  '              2 Sbb Scc Cbc k^2 l^2 ',
     +  ' FW^2= Var(SQ)(P/SQ)^2 {P=180/pi*sqrt(8Ln2) 1.E-03 tan(theta)}'
      return
   11 WRITE(i_out,'(A/A/A/a/a/a/a/a/a/a/)')
     +  ' => Strain in Monoclinic lattices (001-> 2-fold axis)',
     +  '    8 strain parameters corresponding to :',
     +  '    Saa=Sigma(A), Sbb, Scc, Sdd, Cab, Cac, Cbc, Ccd=Corr(C,D)',
     +  '  ->     SQ = A h^2+ B k^2 + C l^2  + D hk ',
     +  '  -> Var(SQ)= Saa^2 h^4 + Sbb^2 k^4 + Scc^2 l^4 + (Sdd hk)^2+',
     +  '              2 Saa Sbb Cab h^2 k^2 +',
     +  '              2 Saa Scc Cac h^2 l^2 +',
     +  '              2 Sbb Scc Cbc k^2 l^2 ',
     +  '              2 Scc Sdd Ccd h k l^2 ',
     +  ' FW^2= Var(SQ)(P/SQ)^2 {P=180/pi*sqrt(8Ln2) 1.E-03 tan(theta)}'
      return
      END
C************************************************************************
      SUBROUTINE COMSHIF(KIND)
      include 'ful0.inc'
      GOTO(100,200,300,400),iabs(kind)
  100 continue
      WRITE(i_out,'(a/a/a/a/a/)')
     +' => Uniaxial shifts along a direction [S1,S2,S3] ',
     +'     Shift of Bragg reflections of the form: ',
     +'             Shv = Shift1 * cos(Phi)    (Ishif= 1)',
     +'          or Shv = Shift1 * sin(Phi)    (Ishif=-1)',
     +'     Where Phi is the angle: between [hkl] and [S1,S2,S3]'
      return
  200 continue
      WRITE(i_out,'(a/a/a/a/)')
     +' =>  Shift of Bragg reflections of the form (Ishif= 2): ',
     +'        Shv = coeff(hkl) * Shift1 * tan(theta)  ',
     +'     Where coeff(hkl) is a user-given coefficient for each',
     +'     reflection read in CODFILn.hkl'
      return
  300 continue
  400 continue
      WRITE(i_out,'(a,i3)')
     +  ' ==> Sorry, no shift model available! ',KIND
      RETURN
      END
C************************************************************************
      SUBROUTINE COMasy(KIND)
      include 'ful0.inc'
      GOTO(100,200,300,400),iabs(kind)
  100 continue
  200 continue
  300 continue
  400 continue
      WRITE(i_out,'(a,i3)')
     + ' ==> Sorry, no hkl-dependent asymmetry model is
     + available! ', KIND
      RETURN
      END
C************************************************************************
C   HKL-dependent asymmetry and shift for stacking faults and other
C   defects can be modeled by using up to three parameters stored
C   in AS=PAR(NPHT,38), and Shift1=PAR(NPHT,39) and Shift2=PAR(NPHT,40)
C----6----------------------------------------------------------------12
      SUBROUTINE SHIFHKL(IX,ICX,H,K,L,IP,SHV,DSHV,SQV,AL)
      INCLUDE 'ful0.inc'
      INCLUDE 'ful1.inc'
      INCLUDE 'ful2.inc'
      DIMENSION DSHV(2),HH(3),AL(3,3)
      REAL K,L
c     DATA RAD/57.29577951/,G/0.017453293/
      SHV=0
      DO 1 J=1,2
   1  DSHV(J)=0.0
      IF(ISHIF(IP).EQ.0) RETURN
      GOTO(100,200,300,400),iabs(ISHIF(IP))
c-------------------------------------------------------------------
c Shift of Bragg reflections of the form Shv= K * cos(q) or / K * sin(q)
c-------------------------------------------------------------------
  100 DS=0.0
      HH(1)=H
      HH(2)=K
      HH(3)=L
      DO I=1,3
       DO J=I,3
        DS=DS+AL(I,J)*(PSH(IP,I)*HH(J)+PSH(IP,J)*HH(I))/2.
       end do
      end do
      DS=abs(DS)/SQRT(PSH(IP,4)*SQV)
      IF(IShif(IP).EQ.-1) THEN
       DS=SQRT(1.-DS*DS)
      ENDIF
       Shv=DS*PAR(IP,39)
       Dshv(1)=DS
      return
  200 sin2=SQV/4.*LAMDA(ICX)*LAMDA(ICX)
      tant=sqrt(sin2/(1.0-sin2))
      DS=PHASEN(IX)*TANT
       Shv=DS*PAR(IP,39)
       Dshv(1)=DS
      return
  300 RETURN
  400 RETURN
      END
C----6----------------------------------------------------------------12
C----6----------------------------------------------------------------12
      SUBROUTINE ASYMHKL(IX,ICX,H,K,L,IP,ASV,DASV,SQV,AL)
      INCLUDE 'ful0.inc'
      INCLUDE 'ful1.inc'
      INCLUDE 'ful2.inc'
c     DIMENSION DSHV(2),HH(3),AL(3,3)
      DIMENSION AL(3,3)
c     REAL K,L
c     DATA RAD/57.29577951/,G/0.017453293/
      ASV=0
      DASV=0.0
      IF(IASV(IP).EQ.0) RETURN
      GOTO(100,200,300,400),iabs(IASV(IP))
  100 RETURN
  200 RETURN
  300 RETURN
  400 RETURN
      END
C*****************************************************************************
C  MAGNETIC INTENSITY FOR REFLECTION NN-> HNN(1),HNN(2),HNN(3)
C********1*********2*********3*********4*********5*********6*********7********
      SUBROUTINE MAGMOD(NN,IPH,IRL,N,IOF,ICENT,IH,HNN,SSNN,FNN,TAV,DERI)
C
c
c     This example of MAGMOD corresponds to the direct space description
c     of a conical magnetic structure
C
c     The magnetic intensity is given by the following formula:
c
c
c Fm**2 =(p.sinw)**2 . Sum(i){ m(i) f(H,i) cos(bi) exp(2pi.i.H ri)}
c Fm**2 =p**2.(1+cosw**2)/4 . Sum(i){ m(i) f(H+-k,i) sin(bi) exp(2pi.i.H ri-+fi)}
c
c     It is assumed that no symmetry operation is accepted for this
c     description. All the magnetic atoms within a primitive nuclear cell
c     must be given explicitely.
c
c-------------------------------------------------------------------------
      INCLUDE 'ful0.inc'
      INCLUDE 'ful1.inc'
      INCLUDE 'ful2.inc'
      LOGICAL SKIP
      DIMENSION HNN(3),IH(3)
      DIMENSION COSAR(NATM),SINAR(NATM),TEMP(NATM),ca(NATM),cb(NATM)
      DIMENSION DERI(MSZ),ccs(NATM)
      DIMENSION COSA(3),SIDE(3),SINA(3)
      Integer  Mod6
      common/SymOpda/Mod6(36,3,3)
C************************************************************************
      DATA RAD/0.0174532925/,TPI/6.28318530718/
C      Change the sign of the magnetic phase when the current scattering
c      vector correspond to -k.
      isig=-1
      IF((IHKL(NN).GT.NVK(IPH)/2).AND.(GENK(IPH).EQ.'Y')) ISIG=1
      DO 1 I=1,3                   !Cell parameters a,b,c
      SIDE(I)=SAVE(IPH,I)
      COSA(I)=COS(SAVE(IPH,I+3)*RAD)
      SINA(I)=SQRT(1.0-COSA(I)*COSA(I))
    1 continue
      IF(NSCAT.LT.0) RSTAR=2000.0*SQRT(SSNN)
c
c  Calculate the crystallographic components of the cone axis ONLY the first
c  time the subroutine is called in the current cycle. The derivatives of the
c  cone components w.r.t. the polar angles are also calculated.
c
      if(ifirst(IPH,2).eq.0) then
       Phi_h(IPH)=xl(iof+1,9)*rad
       Theta_h(iph)=xl(iof+1,10)*rad
       CMX=cos(phi_h(iph))*sin(theta_h(iph))
       CMY=sin(phi_h(iph))*sin(theta_h(iph))
       cone(iph,1)=(CMx-CMy*COSA(3)/SINA(3))/SIDE(1)
       cone(iph,2)=CMy/SINA(3)/SIDE(2)
       cone(iph,3)=cos(theta_h(iph))/SIDE(3)
       do i=1,n
        ipiof=iof+i
        if(lp(ipiof,8).ne.0) then
         XL(ipiof,8)=AMOD(XL(ipiof,8),1.0)
        endif
       end do
       ifirst(IPH,2)=1
       if(LP(IOF+1,9).NE.0) THEN                    !Derivative w.r.t. Phi_h
        dcon1(iph,1)=-rad*(CMy+CMx*COSA(3)/SINA(3))/SIDE(1)
        dcon1(iph,2)=rad*CMx/SINA(3)/SIDE(2)
        dcon1(iph,3)=0.0
       endif
       if(LP(IOF+1,10).NE.0) THEN                   !Derivative w.r.t. Theta_h
        cmx=cos(phi_h(iph))*cos(theta_h(iph))
        cmy=sin(phi_h(iph))*cos(theta_h(iph))
        dcon2(iph,1)=rad*(CMx-CMy*COSA(3)/SINA(3))/SIDE(1)
        dcon2(iph,2)=rad*CMy/SINA(3)/SIDE(2)
        dcon2(iph,3)=-rad*sin(theta_h(iph))/SIDE(3)
       endif
        if(icent.ne.1) icent=1
        if(irl.ne.0) irl=0
      endif
c
c Calculation of the orientation factor for the magnetic intensity
c
      CW=0.
      ss=2.0*sqrt(SSNN)
      DO  I=1,3
       CW=CW+CONE(IPH,I)*HNN(I)
      END DO
      cw=cw/ss
      CW2=CW*CW

      if(IHKL(NN).eq.0) then
       q2=ABS(1.-CW2)
      else
       q2=(1.0+cw2)/4.0
      endif
C
C  Trigonometric part of structure factor
C
C  COSAR(i,j)=Cos{2pi*[H.r(i)+Phas(i)*isig]}
C  SINAR(i,j)=Sin{2pi*[H.r(i)+Phas(i)*isig]}
C
      AF=0.0
      BF=0.0
      DO 2 I=1,N       !Loop over all atoms
      IPIOF=I+IOF
      IF(IHKL(NN).eq.0) then
        arg=0.0
        ccs(i)=cos(xl(ipiof,7)*rad)
       else
        ARG=XL(ipiof,8)*isig     !Magnetic phase added for satellites
        ccs(i)=sin(xl(ipiof,7)*rad)
      end if
        DO J=1,3
         ARG=IH(J)*XL(IPIOF,J)+ARG
        end do
         ARG=TPI*ARG     ! ((hkl)+k){R,t}(xyz)
         COSAR(I)=COS(ARG)
         SINAR(I)=SIN(ARG)
         TEMP(I)=EXP(-XL(IPIOF,4)*SSNN)
         NI=PTR(IPIOF,2)
       IF(NSCAT.LT.0.AND.NI.LE.IABS(NSCAT)) THEN
        CALL SFINT(RSTAR,NI,FFX)
       ELSE
      FFX=AC(7,NI)
      DO  II=1,5,2
       FFX=FFX+AC(II,NI)*EXP(-AC(II+1,NI)*SSNN)  !Form factor value for Q=H+k
      end do
       ENDIF
      temp(i)=0.2696*FFX*TEMP(I)*XL(IPIOF,5)     ! 0.2696.f(Q).Temp(i).Occ
      ca(i)=temp(i)*cosar(i)*xl(ipiof,6)*ccs(i)
      cb(i)=temp(i)*sinar(i)*xl(ipiof,6)*ccs(i)
      AF=AF+ca(i)
      BF=BF+cb(i)
    2 CONTINUE                       !End loop over atoms I-index
      GFF=AF*AF+BF*BF
      FNN=Q2*GFF*TAV
C     write(*,'(3i3,1x,i1,g15.4)')
C    +       (ih(j),j=1,3),ihkl(nn), FNN
c
      IF(MAXS.EQ.0) RETURN
c
C-----CALCULATE DERIVATIVES
c
      TAQ=2.0*TAV*Q2
c-------------------------------------------------------------------------
      DO 3   I=1,N            !Loop over Natoms
       IPIOF=I+IOF
      DO 4  J=1,10         !Loop over parameters up to Mphas
       LPIJ=LP(IPIOF,J)
       SKIP=.FALSE.
       IF(J.GT.8.AND.I.NE.1) SKIP=.TRUE.
       IF(LPIJ.EQ.0.OR.SKIP)GOTO 4    !If not refined goto 4

       IF(J.GT.3) GOTO 10       !Derivatives of FNN w.r.t. x,y,z
       DER=TAQ*TPI*IH(J)*(-AF*CB(I)+BF*CA(I))
       goto 5

  10  IF(J.GT.4) GOTO 11       !Derivative of FNN w.r.t. Biso
       DER=-TAQ*SSNN*(AF*CA(I)+BF*CB(I))
       goto 5

  11  IF(J.GT.6) GOTO 12       !Derivatives of FNN w.r.t. Occ & Moment
       DER=TAQ*(AF*CA(I)+BF*CB(I))/XL(IPIOF,J)
      goto 5

  12  IF(J.GT.7) GOTO 13       !Derivative of FNN w.r.t. the angle of the cone
       DER=TAQ*(AF*Cosar(I)+BF*sinar(I))*temp(i)*XL(IPIOF,6)
       IF(IHKL(NN).EQ.0) THEN
        DER=-DER*SIN(XL(IPIOF,7)*RAD)*RAD
       ELSE
        DER=DER*COS(XL(IPIOF,7)*RAD)*RAD
       ENDIF
      goto 5

  13  IF(J.GT.8) GOTO 14       !Derivative of FNN w.r.t. the phase angle
       DER=TAQ*TPI*Isig*(-AF*CB(I)+BF*CA(I))
      goto 5

  14  IF(J.GT.9) GOTO 15       !Derivative of FNN w.r.t. Phi_h
       x=0.0
       do l=1,3
        x=x+ dcon1(iph,l)*hnn(l)
       end do
       x=x/ss                  ! Dcosw/DPhi_h
       if(IHKL(NN).EQ.0) then
        DER=0.5*CW*X*GFF*TAV
       else
        DER=2.0*CW*X*GFF*TAV
       endif
      goto 5

  15  continue !J=10           !Derivative of FNN w.r.t. Theta_h
       x=0.0
       do l=1,3
        x=x+ dcon2(iph,l)*hnn(l)
       end do
       x=x/ss                  ! Dcosw/DTheta_h
       if(IHKL(NN).EQ.0) then
        DER=0.5*CW*X*GFF*TAV
       else
        DER=2.0*CW*X*GFF*TAV
       endif

   5  DERI(LPIJ)=A(IPIOF,J)*DER+DERI(LPIJ)
c-------------------------------------------------------------------------
   4  CONTINUE         !End loop over parameters (J-index)
   3  CONTINUE         !End loop over atoms (I-index)
c-------------------------------------------------------------------------
C     write(*,'(3i3,1x,i1,4g15.4)')
C    +       (ih(j),j=1,3),ihkl(nn), FNN, deri(4),deri(5),deri(6)
      RETURN
      END
C*****************************************************************************
C************************************************************************
      SUBROUTINE COMMENTS5(K)
      include 'ful0.inc'
C
C   This subroutines write in file *.out information given by the user
C   concerning his(her) explanation about the use of parameters in MAGMOD
C   K: No. of the current Phase for which JBT=5
      WRITE(i_out,'(a,i3)')
     + ' ==> Comments on Magnetic Refinement of phase:',
     +K
      WRITE(i_out,'(a)')
     + '    Magnetic structure factor calculated for a conical'
      WRITE(i_out,'(a)')'    structure following the reference:'
      WRITE(i_out,'(a)')' '
      WRITE(i_out,'(a)')
     + '  -> J.M.Hasting & L.M.Corliss, PhysRev 126(2),556 (1962)'
      WRITE(i_out,'(a)')' '
      WRITE(i_out,'(a)')
     +'  -> P6 is the magnetic moment of the atom (in Bohr magnetons)'
      WRITE(i_out,'(a)')
     +'  -> P7 is the half-angle cone of the atom (degrees)'
      WRITE(i_out,'(a)')'  -> P8 is the magnetic phase of the atom '
      WRITE(i_out,'(a)')'  -> P9    of the first atom correspond to Phi'
      WRITE(i_out,'(a)')
     + '  -> P10   of the first atom correspond to Theta'
      WRITE(i_out,'(a)')
     +'  (Phi,Theta) are the spherical angles(degrees) of the cone axis'
      WRITE(i_out,'(a)')' '
      WRITE(i_out,'(a/a/a/a/a)')
     +'   It is recommended to generate the reflections using L-1 as ',
     +'   Space group symbol and use for conical structures the value',
     +'   IRF=0 (Generates satellites+ fundamentals). For a pure helix',
     +'   IFR=-1 (only satellites are generated)',
     +'   L=P,A,B,C,F,I,R is the lattice type -> Primitive or Centered)'
      WRITE(i_out,'(a/)')'   '
      RETURN
      END
C************************************************************************
      SUBROUTINE FORM_FACTOR(IPH,NI,IPIOF,SSNN,H,XI,FFX,BNI,DXI,DERIF)
C-------------------------------------------------------------------------
C   Subroutine to calculate an user-supplied form-factor
C   The parameters that can be used for describing the form-factor are
C   stored in TWO arrays XI(19->32) and PAR(IPH,j+LASTMPAR), j=1,Ifurt(IPH)
C   IPH:index of the phase cannot be changed within the subroutine
C   NI: Integer used for getting the type (1,2,3,4,or 5) of form-factor
C   IPIOF: Integer indexing the object in the atom list
C   SSNN: sin(theta)/Lambda  for the current reflection H=(h,k,l)
C    The following parameters should be provided in the subroutine
C   FFX : value of the form factor
C   DXI(ipiof,j): derivatives of the form factor w.r.t. parameters XI(j)
C   DERIF(j): derivatives of the form factor w.r.t. parameters PAR(IPH,j+LASTMPAR)
C-------------------------------------------------------------------------
      INCLUDE 'ful0.inc'
      INCLUDE 'ful1.inc'
      INCLUDE 'ful2.inc'
      DIMENSION H(3),DERIF(MPAR),XI(nat_p),DXI(NATS,nat_p)
      CHARACTER*4 LABELFF
      REAL*8 PI,Q,r,qr
      PARAMETER (PI=3.141592653589793d0)
      BNI=0.0
      LABELFF=NAM(-NI)
c------------------------------------Simple 0-order Bessel function
      IF(LABELFF.EQ.'SPHS') THEN
c------------------------------------Spherical surface
       r= XL(Ipiof,19)
       q=4.0*pi*sqrt(SSNN)
       qr=q*r
       FFX=sin(qr)/qr
       if(MAXS.eq.0) return
       if(LP(ipiof,19).ne.0) then
        DXI(ipiof,19)=cos(qr)/qr - ffx/r
       endif
       return
c-----------------------------------------------------------------------
      ELSEIF(LABELFF.EQ.'SPHE') THEN
       FFX=0.0
       if(MAXS.eq.0) return
       do j=19,Nat_p
        K=LP(ipiof,j)
        if(K.ne.0) then
         DXI(ipiof,J)=0.0    !Put here the true derivatives
        endif
       end do
       do j=1,ifurt(IPH)
        K=LPAR(iph,j+LASTMPAR)
        if(K.ne.0) then
         DERIF(J)=0.0    !Put here the true derivatives
        endif
       end do
      RETURN
      ELSEIF(LABELFF.EQ.'ELLI') THEN
       FFX=0.0
      ELSEIF(LABELFF.EQ.'DISK') THEN
       FFX=0.0
      ELSEIF(LABELFF.EQ.'SASH') THEN
       FFX=0.0
      ELSEIF(LABELFF.EQ.'TORE') THEN
       FFX=0.0
      ELSEIF(LABELFF.EQ.'HARM') THEN
       FFX=0.0
      ELSE
       FFX=0.0
       return
      ENDIF
      END
